/* SETHMR.c */

#pragma strings(readonly)
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#define INCL_BASE
#define INCL_ERRORS
#include <os2.h>
typedef USHORT WORD;
typedef ULONG DWORD;
#include <exe386.h>

/*  */
#pragma pack(1)
struct StubHeader_t
{
    USHORT usID;              /* == 0x5a4d */
    USHORT ausContents1[11];
    USHORT usRelOff;          /* == 30 or > 32 */
    USHORT ausContents2[17];
    ULONG ulNewHeaderOff;
};
#pragma pack()

/*  */
int main(int argc,
         char* argv[])
{
    HFILE hfModule;
    ULONG ulWork;
    FILESTATUS3 xfs3;
    PVOID pvBuffer;
    struct e32_exe* pxLXHeader;
    struct o32_obj* pxObjTable;
    int nBit;

    if(argc < 2)
    {
        (void)printf("** DANGER **  for testing purpose only  ** DANGER **\n"
                     "\n"
                     "SETHMR modifies 1st object attributes field of specified executable module.\n"
                     "ex. SETHMR foo.dll 16\n"
                     "      sets bit 16 of foo.dll's 1st object attributes field.\n"
                     "\n"
                     "SETHMR: filename required.\n");
        return -1;
    }
    if(DosOpen((PSZ)argv[1],
               &hfModule,
               &ulWork,
               (ULONG)0,
               FILE_NORMAL,
               OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
               OPEN_FLAGS_FAIL_ON_ERROR | OPEN_FLAGS_NOINHERIT | OPEN_SHARE_DENYNONE | OPEN_ACCESS_READWRITE,
               (PEAOP2)NULL) != NO_ERROR)
    {
        (void)printf("SETHMR: \"%s\" cannot be opened.\n",
                     (const char*)argv[1]);
        return -1;
    }
    (VOID)DosQueryFileInfo(hfModule,
                           FIL_STANDARD,
                           (PVOID)&xfs3,
                           (ULONG)sizeof(xfs3));
    (VOID)DosAllocMem(&pvBuffer,
                      xfs3.cbFile,
                      PAG_READ | PAG_WRITE | PAG_COMMIT);
    if(DosRead(hfModule,
               pvBuffer,
               xfs3.cbFile,
               &ulWork) != NO_ERROR ||
       ulWork != xfs3.cbFile)
    {
        (void)printf("SETHMR: \"%s\" cannot be read.\n",
                     (const char*)argv[1]);
        (VOID)DosFreeMem(pvBuffer);
        (VOID)DosClose(hfModule);
        return -1;
    }
    pxLXHeader = (struct e32_exe*)NULL;
    switch(((PUSHORT)pvBuffer)[0])
    {
        case (USHORT)0x5a4d:  /* 'MZ' */
            if(((struct StubHeader_t*)pvBuffer)->usRelOff >= (USHORT)sizeof(struct StubHeader_t))
            {
                pxLXHeader = (struct e32_exe*)&((PUCHAR)pvBuffer)[((struct StubHeader_t*)pvBuffer)->ulNewHeaderOff];
                if(((PUSHORT)pxLXHeader)[0] != (USHORT)0x584c)  /* 'LX' */
                    pxLXHeader = (struct e32_exe*)NULL;
            }
            break;
        case (USHORT)0x584c:  /* 'LX' */
            pxLXHeader = (struct e32_exe*)pvBuffer;
            break;
    }
    if(pxLXHeader == (struct e32_exe*)NULL ||
       pxLXHeader->e32_objcnt == (unsigned long)0)
    {
        (void)printf("SETHMR: \"%s\" is not loadable module.\n",
                     (const char*)argv[1]);
        (VOID)DosFreeMem(pvBuffer);
        (VOID)DosClose(hfModule);
        return -1;
    }
    pxObjTable = (struct o32_obj*)&((PUCHAR)pxLXHeader)[pxLXHeader->e32_objtab];
    pxObjTable->o32_flags &= (unsigned long)0x0000ffff;
    if(argc > 2)
    {
        nBit = atoi((const char*)argv[2]);
        if(nBit >= 0 &&
           nBit < 32)
            pxObjTable->o32_flags |= (unsigned int)1 << nBit;
    }
    (VOID)DosSetFilePtr(hfModule,
                        (LONG)0,
                        FILE_BEGIN,
                        &ulWork);
    if(DosWrite(hfModule,
                pvBuffer,
                xfs3.cbFile,
                &ulWork) != NO_ERROR ||
       ulWork != xfs3.cbFile)
    {
        (void)printf("SETHMR: \"%s\" cannot be written.\n",
                     (const char*)argv[1]);
        (VOID)DosFreeMem(pvBuffer);
        (VOID)DosClose(hfModule);
        return -1;
    }
    (void)printf("SETHMR: \"%s\" is modified.\n",
                 (const char*)argv[1]);
    (VOID)DosFreeMem(pvBuffer);
    (VOID)DosClose(hfModule);
    return 0;
}

